<?php

namespace Drupal\entity_generic\Controller;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\OpenModalDialogCommand;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Entity\Entity\EntityFormDisplay;
use Drupal\Core\Entity\Entity\EntityViewDisplay;
use Drupal\Core\Entity\EntityFormBuilderInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityViewBuilderInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\entity_generic\Entity\GenericTypeInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Controller routines for entity routes.
 */
class GenericModalController extends GenericController {

  /**
   * The entity form builder.
   *
   * @var \Drupal\Core\Entity\EntityFormBuilderInterface
   */
  public $entityFormBuilder;

  /**
   * @param \Drupal\Core\Datetime\DateFormatterInterface $date_formatter
   *   The date formatter service.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer service.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   * @param \Drupal\Core\Entity\EntityFormBuilderInterface $entity_form_builder
   */
  public function __construct(DateFormatterInterface $date_formatter, RendererInterface $renderer, RequestStack $request_stack, EntityFormBuilderInterface $entity_form_builder) {
    parent::__construct($date_formatter, $renderer, $request_stack);
    $this->entityFormBuilder = $entity_form_builder;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('date.formatter'),
      $container->get('renderer'),
      $container->get('request_stack'),
      $container->get('entity.form_builder')
    );
  }

  /**
   * Callback for composing the new entity.
   *
   * @param $entity_class
   * @param array $values
   * @param array $args
   *
   * @return \Drupal\Core\Ajax\AjaxResponse
   */
  public function composeEntity($entity_class, array $values, array $args = []) {
    return $entity_class::create($values);
  }

  /**
   * Callback to view the entity using modal form.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *
   * @return \Drupal\Core\Ajax\AjaxResponse
   */
  public function viewGenericEntityModal(EntityInterface $entity) {
    $request_parameters = \Drupal::request()->query->all();

    // Compose the modal view.
    $view_mode = isset($request_parameters['view_mode']) ? $request_parameters['view_mode'] : 'default';
    $view_builder = $this->entityTypeManager->getViewBuilder($entity->getEntityTypeId());
    $view = $view_builder->view($entity, $view_mode);

    // AJAX response.
    $response = new AjaxResponse();

    // Add an AJAX command to open a modal dialog with the form as the content.
    $response->addCommand(new OpenModalDialogCommand($entity->getLabel(), $view, ['width' => $request_parameters['width'], 'height' => $request_parameters['height']]));

    return $response;
  }

  /**
   * Callback for adding the entity using modal form.
   *
   * @param array $args
   *
   * @return \Drupal\Core\Ajax\AjaxResponse
   */
  public function addGenericEntityModal(array $args = []) {
    $request_parameters = \Drupal::request()->query->all();
    $route_parameters = \Drupal::routeMatch()->getParameters()->all();
    foreach ($route_parameters as $route_parameter_name => $route_parameter) {
      if ($route_parameter instanceof GenericTypeInterface) {
        $entity_bundle_info = $this->entityTypeManager->getDefinition($route_parameter->bundle());
        $entity_type = $entity_bundle_info->getBundleOf();
        $entity_type_info = $this->entityTypeManager->getDefinition($entity_type);
        $entity_class = $entity_type_info->getClass();
        break;
      }
    }

    // New entity.
    $entity = $this->composeEntity($entity_class, ['type' => $route_parameter->id()], $request_parameters);

    // Get the modal form using the form builder.
    $form_state_additions = [
      'modal_form_display' => EntityFormDisplay::collectRenderDisplay($entity, 'modal'),
    ];
    $form_state_additions += $request_parameters;
    $form = $this->entityFormBuilder->getForm($entity, 'modal', $form_state_additions);

    // AJAX response.
    $response = new AjaxResponse();

    // Add an AJAX command to open a modal dialog with the form as the content.
    $response->addCommand(new OpenModalDialogCommand($this->t('Add') . ' ' . $entity->getEntityType()->getLabel()->__toString(), $form, ['width' => '800']));

    return $response;
  }

  /**
   * Callback for editing the entity using modal form.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *
   * @return \Drupal\Core\Ajax\AjaxResponse
   */
  public function editGenericEntityModal(EntityInterface $entity) {
    $request_parameters = \Drupal::request()->query->all();

    // Get the modal form using the form builder.
    $form_mode = isset($request_parameters['form_mode']) ? $request_parameters['form_mode'] : 'modal';
    $form_state_additions = [
      'modal_form_display' => EntityFormDisplay::collectRenderDisplay($entity, $form_mode),
    ];
    $form_state_additions += $request_parameters;
    $form = $this->entityFormBuilder->getForm($entity, $form_mode, $form_state_additions);

    // AJAX response.
    $response = new AjaxResponse();

    // Add an AJAX command to open a modal dialog with the form as the content.
    $response->addCommand(new OpenModalDialogCommand($this->t('Edit') . ' ' . $entity->getEntityType()->getLabel()->__toString(), $form, ['width' => '800']));

    return $response;
  }

  /**
   * Callback for editing the entity using modal form.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *
   * @return \Drupal\Core\Ajax\AjaxResponse
   */
  public function toggleStatusModal(EntityInterface $entity) {
    $request_parameters = \Drupal::request()->query->all();

    // Get the modal form using the form builder.
    $form_mode = isset($request_parameters['form_mode']) ? $request_parameters['form_mode'] : 'status_modal';
    $form_state_additions = [
      'modal_form_display' => EntityFormDisplay::collectRenderDisplay($entity, $form_mode),
    ];
    $form_state_additions += $request_parameters;
    $form = $this->entityFormBuilder->getForm($entity, $form_mode, $form_state_additions);

    // AJAX response.
    $response = new AjaxResponse();

    // Add an AJAX command to open a modal dialog with the form as the content.
    $response->addCommand(new OpenModalDialogCommand($this->t('Change status'), $form, ['width' => '800']));

    return $response;
  }

  /**
   * Callback for deleting the entity using modal form.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *
   * @return \Drupal\Core\Ajax\AjaxResponse
   */
  public function deleteGenericEntityModal(EntityInterface $entity) {
    $request_parameters = \Drupal::request()->query->all();

    // Get the modal form using the form builder.
    $form_mode = isset($request_parameters['form_mode']) ? $request_parameters['form_mode'] : 'delete_modal';
    $form_state_additions = [
      'modal_form_display' => EntityFormDisplay::collectRenderDisplay($entity, $form_mode),
    ];
    $form_state_additions += $request_parameters;
    $form = $this->entityFormBuilder->getForm($entity, $form_mode, $form_state_additions);

    // AJAX response.
    $response = new AjaxResponse();

    // Add an AJAX command to open a modal dialog with the form as the content.
    $response->addCommand(new OpenModalDialogCommand($this->t('Delete ' . $entity->getEntityType()->getLabel()->__toString()), $form, ['width' => '800']));

    return $response;
  }

}
