<?php

namespace Drupal\entity_generic\Plugin\Field\FieldFormatter;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityManagerInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\Plugin\Field\FieldFormatter\StringFormatter;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'entity_generic_string' formatter.
 *
 * @FieldFormatter(
 *   id = "entity_generic_string",
 *   label = @Translation("Plain text (entity generic)"),
 *   field_types = {
 *     "string",
 *     "uri",
 *   },
 *   quickedit = {
 *     "editor" = "plain_text"
 *   }
 * )
 */
class GenericStringFormatter extends StringFormatter {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    $options = parent::defaultSettings();

    $options['link_modal_to_entity'] = FALSE;
    $options['link_modal_view_mode'] = 'default';
    $options['link_modal_width'] = 800;
    $options['link_modal_height'] = 600;
    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $form = parent::settingsForm($form, $form_state);

    $entity_type = $this->entityManager->getDefinition($this->fieldDefinition->getTargetEntityTypeId());

    $form['link_to_entity']['#states'] = [
      'disabled' => [
        ':input[name="options[settings][link_modal_to_entity]"]' => ['checked' => TRUE],
      ],
    ];

    $form['link_modal_to_entity'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Modal Link to the @entity_label', ['@entity_label' => $entity_type->getLabel()]),
      '#default_value' => $this->getSetting('link_modal_to_entity'),
      '#states' => [
        'disabled' => [
          ':input[name="options[settings][link_to_entity]"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $entityDisplayRepository = \Drupal::service('entity_display.repository');
    $options = $entityDisplayRepository->getViewModeOptions($entity_type);
    $form['link_modal_view_mode'] = [
      '#type' => 'select',
      '#title' => $this->t('View mode'),
      '#options' => $options,
      '#default_value' => $this->getSetting('link_modal_view_mode')
    ];

    $form['link_modal_width'] = [
      '#type' => 'number',
      '#title' => t('Modal window width'),
      '#field_suffix' => t('px'),
      '#default_value' => $this->getSetting('link_modal_width'),
      '#min' => 1,
      '#states' => [
        'visible' => [
          ':input[name="options[settings][link_modal_to_entity]"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['link_modal_height'] = [
      '#type' => 'number',
      '#title' => t('Modal window height'),
      '#field_suffix' => t('px'),
      '#default_value' => $this->getSetting('link_modal_height'),
      '#min' => 1,
      '#states' => [
        'visible' => [
          ':input[name="options[settings][link_modal_to_entity]"]' => ['checked' => TRUE],
        ],
      ],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];
    if ($this->getSetting('link_modal_to_entity')) {
      $entity_type = $this->entityManager->getDefinition($this->fieldDefinition->getTargetEntityTypeId());
      $summary[] = $this->t('Linked to the @entity_label (will open in modal window)', ['@entity_label' => $entity_type->getLabel()]);
    }
    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];
    $url = NULL;
    if ($this->getSetting('link_to_entity') || $this->getSetting('link_modal_to_entity')) {
      // For the default revision this falls back to 'canonical'.
      $url = $this->getEntityUrl($items->getEntity());
    }

    foreach ($items as $delta => $item) {
      $view_value = $this->viewValue($item);
      if ($url) {

        if ($this->getSetting('link_modal_to_entity')) {
          $url->setOption('query', [
            'width' => $this->getSetting('link_modal_width'),
            'height' => $this->getSetting('link_modal_height'),
            'view_mode' => $this->getSetting('link_modal_view_mode'),
          ]);
        }

        $elements[$delta] = [
          '#type' => 'link',
          '#title' => $view_value,
          '#url' => $url,
        ];

        if ($this->getSetting('link_modal_to_entity')) {
          $elements[$delta]['#attributes']['class'][] = 'use-ajax';
          $elements[$delta]['#attached']['library'][] = 'core/drupal.dialog.ajax';
        }

      }
      else {
        $elements[$delta] = $view_value;
      }
    }
    return $elements;
  }

  /**
   * Gets the URI elements of the entity.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity object.
   *
   * @return \Drupal\Core\Url
   *   The URI elements of the entity.
   */
  protected function getEntityUrl(EntityInterface $entity) {
    if ($this->getSetting('link_modal_to_entity')) {
      return $entity->toUrl('canonical-modal');
    }
    // For the default revision this falls back to 'canonical'.
    return $entity->toUrl('revision');
  }

}
