<?php

namespace Drupal\entity_generic\Routing;

use Drupal\Component\Utility\Unicode;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\Routing\EntityRouteProviderInterface;
use Symfony\Component\Routing\Route;
use Symfony\Component\Routing\RouteCollection;

/**
 * Defines dynamic routes.
 */
class GenericRouteProvider implements EntityRouteProviderInterface {

  /**
   * {@inheritdoc}
   */
  public function getRoutes(EntityTypeInterface $entity_type) {
    $entity_type_id = $entity_type->id();
    $entity_bundle_id = $entity_type->getBundleEntityType();
    $entity_bundle = $entity_bundle_id ? \Drupal::service('entity_type.manager')->getDefinition($entity_bundle_id) : FALSE;

    $route_collection = new RouteCollection();
    $entity_bundle_links = $entity_bundle ? $entity_bundle->get('links') : [];
    $entity_type_links = $entity_type ? $entity_type->get('links') : [];
    $entity_type_callbacks = $entity_type && $entity_type->get('entity_generic') ? $entity_type->get('entity_generic')['callbacks'] : [];

    if ($entity_bundle) {
      // Admin entity bundles.
      if (isset($entity_links['collection'])) {
        $route = (new Route($entity_bundle_links['collection']))
          ->addDefaults([
            '_entity_list' => $entity_bundle_id,
            '_title' => Unicode::ucfirst($entity_bundle->getPluralLabel()),
          ])
          ->setRequirement('_permission', 'administer ' . $entity_type_id)
          ->setOption('_admin_route', TRUE);
        $route_collection->add('entity.' . $entity_bundle_id . '.collection', $route);
      }

      // Add entity bundle.
      if (isset($entity_bundle_links['add-form'])) {
        $route = (new Route($entity_bundle_links['add-form']))
          ->addDefaults([
            '_entity_form' => $entity_bundle_id . '.add',
            '_title' => 'Add ' . $entity_bundle->getSingularLabel(),
          ])
          ->setRequirement('_permission', 'administer ' . $entity_type_id)
          ->setOption('_admin_route', TRUE);
        $route_collection->add('entity.' . $entity_bundle_id . '.add_form', $route);
      }

      // Edit entity bundle.
      if (isset($entity_bundle_links['edit-form'])) {
        $route = (new Route($entity_bundle_links['edit-form']))
          ->addDefaults([
            '_entity_form' => $entity_bundle_id . '.edit',
            '_title' => 'Edit ' . $entity_bundle->getSingularLabel(),
          ])
          ->setRequirement('_permission', 'administer ' . $entity_type_id)
          ->setOption('_admin_route', TRUE);
        $route_collection->add('entity.' . $entity_bundle_id . '.edit_form', $route);
      }

      // Delete entity bundle.
      if (isset($entity_bundle_links['delete-form'])) {
        $route = (new Route($entity_bundle_links['delete-form']))
          ->addDefaults([
            '_entity_form' => $entity_bundle_id . '.delete',
            '_title' => 'Delete ' . $entity_bundle->getSingularLabel(),
          ])
          ->setRequirement('_permission', 'administer ' . $entity_type_id)
          ->setOption('_admin_route', TRUE);
        $route_collection->add('entity.' . $entity_bundle_id . '.delete_form', $route);
      }
    }

    // Admin entity list.
    if (isset($entity_type_links['collection'])) {
      $route = (new Route($entity_type_links['collection']))
        ->addDefaults([
          '_entity_list' => $entity_type_id,
          '_title' => Unicode::ucfirst($entity_type->getPluralLabel()),
        ])
        ->setRequirement('_permission', 'access ' . $entity_type_id . ' overview')
        ->setOption('_admin_route', TRUE);
      $route_collection->add('entity.' . $entity_type_id . '.collection', $route);
    }

    // Handle entities with bundles.
    if ($entity_bundle) {

      // Add entity page.
      if (
        isset($entity_type_links['collection'])
        && isset($entity_type_callbacks['entity.' . $entity_type_id . '.add_page'])
      ) {
        $route = (new Route($entity_type_links['collection'] . '/add'))
          ->addDefaults([
            '_controller' => $entity_type_callbacks['entity.' . $entity_type_id . '.add_page'],
            '_title' => 'Add ' . $entity_type->getSingularLabel(),
            '_bundle_type' => $entity_bundle_id,
          ])
          ->setRequirement('_permission', 'administer ' . $entity_type_id)
          ->setOption('_admin_route', TRUE);
        $route_collection->add('entity.' . $entity_type_id . '.add_page', $route);
      }

      // Add entity.
      if (
        isset($entity_type_links['collection'])
        && isset($entity_type_callbacks['entity.' . $entity_type_id . '.add_entity'])
        && isset($entity_type_callbacks['entity.' . $entity_type_id . '.add_entity_title'])
      ) {
        $route = (new Route($entity_type_links['collection'] . '/add/' . '{' . $entity_bundle_id . '}'))
          ->addDefaults([
            '_controller' => $entity_type_callbacks['entity.' . $entity_type_id . '.add_entity'],
            '_title_callback' => $entity_type_callbacks['entity.' . $entity_type_id . '.add_entity_title'],
          ])
          ->setOption('_admin_route', TRUE)
          ->setOption('operation', 'add')
          ->setOption('parameters',  array(
            $entity_bundle_id => array(
              'type' => 'entity:' . $entity_bundle_id,
              'with_config_overrides' => TRUE,
            ),
          ))
          ->setRequirement('_entity_create_access', $entity_type_id . ':{' . $entity_bundle_id . '}');
        $route_collection->add('entity.' . $entity_type_id . '.add_form', $route);
      }

      // Add entity with modal form.
      if (
        isset($entity_type_links['collection'])
        && isset($entity_type_callbacks['entity.' . $entity_type_id . '.add_entity_modal_title'])
        && isset($entity_type_callbacks['entity.' . $entity_type_id . '.add_entity_modal'])
      ) {
        $route = (new Route($entity_type_links['collection'] . '/add-modal/' . '{' . $entity_bundle_id . '}'))
          ->addDefaults([
            '_controller' => $entity_type_callbacks['entity.' . $entity_type_id . '.add_entity_modal'],
            '_title_callback' => $entity_type_callbacks['entity.' . $entity_type_id . '.add_entity_modal_title'],
          ])
          ->setOption('_admin_route', TRUE)
          ->setOption('operation', 'add')
          ->setOption('parameters',  array(
            $entity_bundle_id => array(
              'type' => 'entity:' . $entity_bundle_id,
              'with_config_overrides' => TRUE,
            ),
          ))
          ->setRequirement('_entity_create_access', $entity_type_id . ':{' . $entity_bundle_id . '}');
        $route_collection->add('entity.' . $entity_type_id . '.add_modal_form', $route);
      }

    }
    // Handle entities without bundles.
    else {

      // Add entity.
      if (isset($entity_type_links['add-form'])) {
        $route = (new Route($entity_type_links['add-form']))
          ->addDefaults([
            '_entity_form' => $entity_type_id . '.add',
            '_title' => 'Add ' . $entity_type->getSingularLabel(),
          ])
          ->setRequirement('_permission', 'create ' . $entity_type_id)
          ->setOption('_admin_route', TRUE);
        $route_collection->add('entity.' . $entity_type_id . '.add_form', $route);
      }

      // Add entity with modal form.
      if (isset($entity_type_links['add-modal-form'])) {
        $route = (new Route($entity_type_links['add-modal-form']))
          ->addDefaults([
            '_controller' => $entity_type_callbacks['entity.' . $entity_type_id . '.add_entity_modal'],
            '_title' => 'Add ' . $entity_type->getSingularLabel(),
          ])
          ->setRequirement('_permission', 'create ' . $entity_type_id)
          ->setOption('_admin_route', TRUE);
        $route_collection->add('entity.' . $entity_type_id . '.add_modal_form', $route);
      }

    }

    // Entity view.
    if (
      isset($entity_type_links['canonical'])
      && isset($entity_type_callbacks['entity.' . $entity_type_id . '.canonical.title'])
    ) {
      $route = (new Route($entity_type_links['canonical']))
        ->addDefaults([
          '_entity_view' => $entity_type_id . '.full',
          '_title_callback' => $entity_type_callbacks['entity.' . $entity_type_id . '.canonical.title'],
        ])
        ->setRequirement($entity_type_id, '\d+')
        ->setRequirement('_entity_access', $entity_type_id . '.view');
      $route_collection->add('entity.' . $entity_type_id . '.canonical', $route);
    }

    // Entity modal view.
    if (
      isset($entity_type_links['canonical-modal'])
      && isset($entity_type_callbacks['entity.' . $entity_type_id . '.canonical.title'])
      && isset($entity_type_callbacks['entity.' . $entity_type_id . '.view_entity_modal'])
    ) {
      $route = (new Route($entity_type_links['canonical-modal']))
        ->addDefaults([
          '_controller' => $entity_type_callbacks['entity.' . $entity_type_id . '.view_entity_modal'],
          '_title_callback' => $entity_type_callbacks['entity.' . $entity_type_id . '.canonical.title'],
        ])
        ->setRequirement($entity_type_id, '\d+')
        ->setRequirement('_entity_access', $entity_type_id . '.view');
      $route_collection->add('entity.' . $entity_type_id . '.canonical_modal', $route);
    }

    // Edit entity.
    if (isset($entity_type_links['edit-form'])) {
      $route = (new Route($entity_type_links['edit-form']))
        ->addDefaults([
          '_entity_form' => $entity_type_id . '.edit',
          '_title' => 'Edit ' . $entity_type->getSingularLabel(),
        ])
        ->setRequirement($entity_type_id, '\d+')
        ->setRequirement('_entity_access', $entity_type_id . '.update');
      $route_collection->add('entity.' . $entity_type_id . '.edit_form', $route);
    }

    // Edit entity with modal form.
    if (
      isset($entity_type_links['edit-modal-form'])
      && isset($entity_type_callbacks['entity.' . $entity_type_id . '.edit_entity_modal'])
    ) {
      $route = (new Route($entity_type_links['edit-modal-form']))
        ->addDefaults([
          '_controller' => $entity_type_callbacks['entity.' . $entity_type_id . '.edit_entity_modal'],
          '_title' => 'Edit ' . $entity_type->getSingularLabel(),
        ])
        ->setRequirement($entity_type_id, '\d+')
        ->setRequirement('_entity_access', $entity_type_id . '.update');
      $route_collection->add('entity.' . $entity_type_id . '.edit_modal_form', $route);
    }

    // Delete entity.
    if (isset($entity_type_links['delete-form'])) {
      $route = (new Route($entity_type_links['delete-form']))
        ->addDefaults([
          '_entity_form' => $entity_type_id . '.delete',
          '_title' => 'Delete ' . $entity_type->getSingularLabel(),
        ])
        ->setRequirement($entity_type_id, '\d+')
        ->setRequirement('_entity_access', $entity_type_id . '.delete');
      $route_collection->add('entity.' . $entity_type_id . '.delete_form', $route);
    }

    // Delete entity with modal form.
    if (
      isset($entity_type_links['delete-modal-form'])
      && isset($entity_type_callbacks['entity.' . $entity_type_id . '.delete_entity_modal'])
    ) {
      $route = (new Route($entity_type_links['delete-modal-form']))
        ->addDefaults([
          '_controller' => $entity_type_callbacks['entity.' . $entity_type_id . '.delete_entity_modal'],
          '_title' => 'Delete ' . $entity_type->getSingularLabel(),
        ])
        ->setRequirement($entity_type_id, '\d+')
        ->setRequirement('_entity_access', $entity_type_id . '.delete');
      $route_collection->add('entity.' . $entity_type_id . '.delete_modal_form', $route);
    }

    // Toggle status.
    if (
      isset($entity_type_links['toggle-status-form'])
      && $entity_type_callbacks['entity.' . $entity_type_id . '.toggle_status']
    ) {
      $route = (new Route($entity_type_links['status-toggle']))
        ->addDefaults([
          '_controller' => $entity_type_callbacks['entity.' . $entity_type_id . '.status_toggle'],
          '_title' => 'Toggle status for ' . $entity_type->getSingularLabel(),
        ])
        ->setRequirement($entity_type_id, '\d+')
        ->setRequirement('_entity_access', $entity_type_id . '.update');
      $route_collection->add('entity.' . $entity_type_id . '.status_toggle', $route);
    }

    // Toggle status with AJAX.
    if (
      isset($entity_type_links['toggle-status-modal-form'])
      && $entity_type_callbacks['entity.' . $entity_type_id . '.toggle_status_modal']
    ) {
      $route = (new Route($entity_type_links['toggle-status-modal-form']))
        ->addDefaults([
          '_controller' => $entity_type_callbacks['entity.' . $entity_type_id . '.toggle_status_modal'],
          '_title' => 'Toggle status for ' . $entity_type->getSingularLabel(),
        ])
        ->setRequirement($entity_type_id, '\d+')
        ->setRequirement('_entity_access', $entity_type_id . '.update');
      $route_collection->add('entity.' . $entity_type_id . '.status_toggle_modal', $route);
    }

    if ($entity_type->isRevisionable()) {
      // Entity revision history.
      if (
        isset($entity_type_links['revision-history'])
        && isset($entity_type_callbacks['entity.' . $entity_type_id . '.revision_history'])
        && is_callable($entity_type_callbacks['entity.' . $entity_type_id . '.revision_history'])
      ) {
        $route = (new Route($entity_type_links['revision-history']))
          ->addDefaults([
            '_title' => 'Revisions',
            '_controller' => $entity_type_callbacks['entity.' . $entity_type_id . '.revision_history'],
          ])
          ->setRequirement($entity_type_id, '\d+')
          ->setRequirement('_entity_access_revision', 'view')
          ->setOption('_admin_route', TRUE);
        $route_collection->add('entity.' . $entity_type_id . '.revision_history', $route);
      }

      // View entity revision.
      if (
        isset($entity_type_links['revision'])
        && isset($entity_type_callbacks['entity.' . $entity_type_id . '.revision'])
        && is_callable($entity_type_callbacks['entity.' . $entity_type_id . '.revision'])
        && isset($entity_type_callbacks['entity.' . $entity_type_id . '.revision.title'])
        && is_callable($entity_type_callbacks['entity.' . $entity_type_id . '.revision.title'])
      ) {
        $route = (new Route($entity_type_links['revision']))
          ->addDefaults([
            '_controller' => $entity_type_callbacks['entity.' . $entity_type_id . '.revision'],
            '_title_callback' => $entity_type_callbacks['entity.' . $entity_type_id . '.revision.title'],
          ])
          ->setRequirement($entity_type_id, '\d+')
          ->setRequirement('_entity_access_revision', 'view');
        $route_collection->add('entity.' . $entity_type_id . '.revision', $route);
      }
    }

    return $route_collection;
  }

}
